"""
This the about file
"""

__all__ = [
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__copyright__",
]

__title__ = "Harambe"
__version__ = "0.0.9"
__summary__ = "A mid stack Flask based framework that put structure in the " \
              "file system. Features: Caching, Mailing, Storage, CSRF, recaptcha, Cors. " \
              "Supports HTML, Markdown and Jade " \
              "and more"
__uri__ = "https://github.com/mardix/harambe"
__author__ = "Mardix"
__email__ = "mardix@pylot.io"
__license__ = "MIT"
__copyright__ = "(c) 2017 %s" % __author__

